<?php
// $Id

/**
 * @file
 * Provide an SimpleSpy style plugin for Views. This file is autoloaded by views.
 */

/**
  * Implementation of views_plugin_style().
  */
class views_simplespy_plugin_style extends views_plugin_style {
  /**
   * Set default options
   */
  function option_definition() {
    $options = parent::option_definition();
    $options['limit'] = array('default' => 4);
    $options['interval'] = array('default' => 4000);
    $options['fadespeed'] = array('default' => 1000);
    $options['slidespeed'] = array('default' => 1000);
    $options['direction'] = array('default' => 'down');
    $options['autopause'] = array('default' => TRUE);
    $options['include-style'] = array('default' => TRUE);
    return $options;
	}

  /**
   * Options form.
   *
   * @param array $form
   * @param array $form_state
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);

    $form['limit'] = array(
      '#type' => 'textfield',
      '#title' => t('Visible Items'),
      '#size' => 5,
      '#maxlength' => 5,
      '#default_value' => $this->options['limit'],
      '#required' => TRUE,
      '#description' => t('The number of items to display at once. This number <em>must be less than items to display</em> for this view and <em>must be higher than 0</em>.'),
    );
    $form['interval'] = array(
      '#type' => 'textfield',
      '#title' => t('Transition time'),
      '#size' => 5,
      '#maxlength' => 5,
      '#default_value' => $this->options['interval'],
      '#field_suffix' => t('Milliseconds'),
      '#required' => TRUE,
      '#description' => t('The time in milliseconds (1000 ms = 1 sec) between each row transition. Note that this will be added to <em>Fading speed</em> and <em>Sliding speed</em> for the total value that will be used.'),
    );
    $form['fadespeed'] = array(
      '#type' => 'textfield',
      '#title' => t('Fading speed'),
      '#size' => 5,
      '#maxlength' => 5,
      '#default_value' => $this->options['fadespeed'],
      '#field_suffix' => t('Milliseconds'),
      '#required' => TRUE,
      '#description' => t('The time in milliseconds for the fading in and out of the elements.'),
    );
    $form['slidespeed'] = array(
      '#type' => 'textfield',
      '#title' => t('Sliding speed'),
      '#size' => 5,
      '#maxlength' => 5,
      '#default_value' => $this->options['slidespeed'],
      '#field_suffix' => t('Milliseconds'),
      '#required' => TRUE,
      '#description' => t('The time in milliseconds for the sliding down or up of the elements.'),
    );
    $form['direction'] = array(
      '#type' => 'radios',
      '#title' => t('Animation direction'),
      '#default_value' => $this->options['direction'],
      '#options' => array(
        'up'    => t('Up'),
        'down'  => t('Down')
      ),
    );
    $form['autopause'] = array(
      '#type' => 'checkbox',
      '#title' => t('Pause on mouse over'),
      '#default_value' => $this->options['autopause'],
      '#description' => t("Whether or not the Spy should pause when the user's mouse hovers the section."),
    );
    $form['include-style'] = array(
      '#type' => 'checkbox',
      '#title' => t("Use the module's default styling"),
      '#default_value' => $this->options['include-style'],
      '#description' => t("If you disable this, the file in the module's directory <em>views-simplespy.css</em> will not be loaded"),
    );
  }

  /**
   * Find out how many items the display is currently configured to show and
   * check limit to be less than this value.
   * 
   * @param mixed $limit
   * @return boolean
   */
  function is_limit_correct($limit) {
    $maxitems = $this->display->handler->get_option('items_per_page');
    if ($maxitems && intval($limit) >= $maxitems) {
      return FALSE;
    }
    else {
      return TRUE;
    }
  }

  /**
   * Validate options form.
   *
   * @param array $form
   * @param array $form_state
   */
  function options_validate(&$form, &$form_state) {
    parent::options_validate($form, $form_state);

    // Validate numeric fields format.
    foreach (array('limit', 'interval', 'fadespeed', 'slidespeed') as $key) {
      $value = $form_state['values']['style_options'][$key];
      if (!preg_match('/^\d+$/', $value) || intval($value) <= 0) {
        $element = $form[$key];
        form_error($element, t('%title must be positive integer value.', array(
          '%title' => $element['#title'],
        )));
      }
    }

    // Check that limit is correct.
    if (!$this->is_limit_correct($form_state['values']['style_options']['limit'])) {
      form_error($form['limit'], t('Visible items number must be less than items to display for this view.'));
    }
  }
  
  /**
   * Modify options on submit.
   * 
   * Convert some values to integers. Not doing this will cause problems with
   * animation speed options, because strings other than "fast" and "slow"
   * mean 600ms for jQuery.animate().
   * 
   * @param array $form
   * @param array $form_state 
   */
  function options_submit(&$form, &$form_state) {
    parent::options_submit($form, $form_state);
    
    foreach (array('limit', 'interval', 'fadespeed', 'slidespeed') as $key) {
      $value = $form_state['values']['style_options'][$key];
      $form_state['values']['style_options'][$key] = intval($value);
    }
  }
  
  /**
   * Validate plugin.
   *
   * @return array
   */
  function validate() {
    $errors = parent::validate();

    if (!$this->is_limit_correct($this->options['limit'])) {
      $errors[] = t('Style @style requires number of visible items less than number of total output items.', array(
        '@style' => $this->definition['title']
      ));
    }

    return $errors;
  }
}
